%
% Calculate the distribution of point charges on a disk
% using Monte Carlo simulation that minimizes the potential energy!
%
% Author:   Chris Theis         01/21/02
%           ctheis@itp.tu-graz.ac.at
%


function [X_Initial, Y_Initial, X, Y, PotEnergyArray] = MC_ParticlePos( InvTemp, Iterations, NrOfParticles )

% particle coordinates
X = zeros( NrOfParticles, 1 );
Y = zeros( NrOfParticles, 1 );
X_New = zeros( NrOfParticles, 1 );
Y_New = zeros( NrOfParticles, 1 );
PotEnergyArray = zeros( Iterations, 1 );
NrOfTries = 0;

% generate random configurations 
[X, Y] = CreateRNDPoints( NrOfParticles );
X_Initial = X;
Y_Initial = Y;


% compute initial potential energy
PotEnergy = 0.0;
for k = 1: NrOfParticles - 1
    for j = k+1:NrOfParticles
        dx_Old = X(k) - X(j);
        dy_Old = Y(k) - Y(j);
        PotEnergy = PotEnergy + 1.0 / sqrt( dx_Old^2 + dy_Old^2);
    end
end

% main Monte Carlo loop

for i = 1:Iterations
    
   % randomly pick a particle
   k = floor( 1 + rand(1) * (NrOfParticles - 1) );
   [X_New, Y_New] = CreateRNDPoints( 1 );  % random displacement of 1 particle
   % calculate delta of the energy
   dV = 0.0;
   for j=1:NrOfParticles
        if( j ~= k)
            dx_Old = X(k) - X(j);
            dy_Old = Y(k) - Y(j);
            dx_New = X_New - X(j);
            dy_New = Y_New - Y(j);
            dV = dV + ( ( 1.0 / sqrt( dx_New^2 + dy_New^2) - 1.0 / sqrt( dx_Old^2 + dy_Old^2) ) );
         end
   end

   % Metropolis acceptance
   if( dV < 0.0 )              % accept anyway because were on the right way
        X(k) = X_New;        % to minimize the potential energy
        Y(k) = Y_New;
        PotEnergy = PotEnergy + dV;
        NrOfTries = NrOfTries + 1;
   else                               % use Metropolis acceptance
        if( rand(1) < exp( -InvTemp * dV) )
            X(k) = X_New;
            Y(k) = Y_New;
            PotEnergy = PotEnergy + dV;
            NrOfTries = NrOfTries + 1;
        end
    end
    
    PotEnergyArray(i) = PotEnergy;
end

AcceptanceRatio = NrOfTries / Iterations;



% helper function to create random coordinates on a disk
function [x, y] = CreateRNDPoints( NrOfPoints )

r1 = rand( NrOfPoints, 1);
r2 = rand( NrOfPoints, 1);

x = sqrt(r1) .* cos( 2.0 * pi .* r2 );
y = sqrt(r1) .* sin( 2.0 * pi .* r2 );
